import 'dart:async';
import 'dart:convert';

import 'package:args/args.dart';
import 'package:shelf/shelf.dart';
import 'package:shelf/shelf_io.dart' as io;
import 'package:shelf/shelf.dart' as shelf;
import 'package:shelf_router/shelf_router.dart' as route;
import 'package:shelf_router/shelf_router.dart';
import 'package:shelf_static/shelf_static.dart';
import 'dart:io';
import 'package:path/path.dart' as path;

Future<void> main(List<String> args) async {
  var parser = ArgParser();
  parser.addOption('host', abbr: 'h', defaultsTo: '0.0.0.0');
  parser.addOption('port', abbr: 'p', defaultsTo: '9203');
  var results = parser.parse(args);
  var host = results['host'] as String;
  var port = int.parse(results['port'] as String);
  print('host: $host');
  print('port: $port');
  startHtml();
  startWebApi(host, port);
  startWebSocket(host, port);
}

void startHtml() {
  var script = Platform.script;
  var filePath = script.toFilePath();
  var split = filePath.split(Platform.pathSeparator);
  var realPath = '';
  var continued = true;
  for (var tempSplit in split) {
    if (tempSplit.isNotEmpty) {
      if (continued) {
        realPath = realPath + Platform.pathSeparator + tempSplit;
      }
      if (tempSplit == '.pub-cache') {
        continued = false;
        break;
      }
    }
  }
  var spyWebPath = '''/hosted/pub.flutter-io.cn/dstackspy-1.0.8/static/web''';
  var htmlPath = realPath + spyWebPath;
  var handlerHtml =
      createStaticHandler(htmlPath, defaultDocument: 'index.html');
  io.serve(handlerHtml, 'localhost', 9201);
  print('dstack-spy已启动，网页端请访问 http://localhost:9201');
}

void startWebApi(String host, int port) async {
  await io.serve(getApiHandler(host, port), 'localhost', 9202);
}

void startWebSocket(String host, int port) async {
  try {
    var server = await HttpServer.bind(host, port);
    print('dstack-spy已启动，webSocket地址 ws://$host:$port/ws');
    await for (HttpRequest req in server) {
      if (req.uri.path == '/ws') {
        // 将一个HttpRequest提升为WebSocket连接
        var socket = await WebSocketTransformer.upgrade(req);
        socket.listen((event) {
          handleMsg(socket, event);
        });
      }
    }
  } catch (e) {
    print(e);
  }
}

Handler getApiHandler(String host, int port) {
  var data = <String, dynamic>{};
  data['host'] = host;
  data['port'] = port;

  final router = Router();
  router.get('/getHost', (Request request) async {
    return Response.ok(
      json.encode(data),
      headers: {
        HttpHeaders.contentTypeHeader: ContentType.json.mimeType,
        'Access-Control-Allow-Origin': '*'
      },
    );
  });
  router.all('/<ignored|.*>', (Request request) {
    return Response.notFound('Page not found');
  });
  return router;
}

var socketMap = {};

void handleMsg(WebSocket socket, String message) {
  print('----------socket接收到消息----------');
  print(message);
  print('----------socket接收到消息----------');
  Map<String, dynamic> msgMap = jsonDecode(message);
  String socketClient = msgMap['socketClient'];
  String messageType = msgMap['messageType'];
  Map data = msgMap['data'];
  if (socketClient != null) {
    socketMap[socketClient] = socket;
  }
  if (messageType != null) {
    WebSocket htmlSocket = socketMap['html'];
    if (htmlSocket != null) {
      Map messageMapToH5 = {};
      messageMapToH5['messageType'] = messageType;
      messageMapToH5['data'] = data;
      var messageJsonToH5 = json.encode(messageMapToH5);
      htmlSocket.add(messageJsonToH5);
    }
  }
}
